﻿
namespace QuickAccess.Browser {
    #region Imported Namespaces

    using System;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.IO;
    using System.Security;
    using System.Windows;
    using GraphicsManager.Commands;
    using GraphicsManager.ViewModels;
    using GraphicsManager.Views;
    using EventAggregator;

    #endregion

    public class ApplicationLauncher {

        private readonly bool _encryptionEnabled = false;

        private ICommandManager _commandManager;

        protected MainWindowViewModel _mainWindowViewModel;
       
        private IEventAggregator _eventAggregator;

        private int _numLogFiles;

        #region Properties

        public IEventAggregator EventAggregator
        {
            get
            {
                if( this._eventAggregator == null ) { this._eventAggregator = EventAggregatorFactory.Instance; }

                return this._eventAggregator;
            }
            set { this._eventAggregator = value; }
        }

        public ICommandManager CommandManager
        {
            get
            {
                if ( this._commandManager == null ) { this._commandManager = CommandManagerFactory.Instance; }

                return this._commandManager;
            }
            set { this._commandManager = value; }
        }

        #endregion

        #region Members

        public void ApplicationStartup( object sender, StartupEventArgs e ) {
            var homeTab = HomeRibbonBuilder.BuildRibbon( false );
            var tabs = new List<ITabViewModel> {homeTab};

            this._mainWindowViewModel = new MainWindowViewModel(
                "QuickAccess",
                tabs,
                new List<IPageViewModel>( ),
                null,
                CommandManagerFactory.Instance.ManagerCommand,
                this.EventAggregator
            ) { ShowTabHeaderCloseButton = false };

            // TODO: Make sure it passes successfully instead of assuming
            this._mainWindowViewModel.Initialize( );

            var mainWindowView = this.CreateMainWindowView( this._mainWindowViewModel );

            if ( mainWindowView != null ) { this.ShowMainWindow( mainWindowView ); }

            AppDomain.CurrentDomain.ProcessExit += ( o, args ) => this._mainWindowViewModel.Dispose( );
        }

        protected virtual int IsFileLocked( string directory, string name ) {
            var combinedPath = Path.Combine( directory, name );
            var fileInfo = new FileInfo( combinedPath );
            FileStream stream = null;

            try {
                stream = fileInfo.Open( FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.None );
            } catch ( SecurityException ex ) {
                // Don't have permissions
                return 2;
            } catch ( UnauthorizedAccessException ex ) {
                // Is directory or is readonly
                return 3;
            } catch ( IOException ex ) {
                //the file is unavailable because it is:
                //still being written to
                //or being processed by another thread
                return 1;
            } finally {
                if ( stream != null )
                    stream.Close( );
            }

            //file is not locked
            return 0;
        }

        public virtual MainWindowView CreateMainWindowView( MainWindowViewModel mainWindowViewModel ) {
            return new MainWindowView {DataContext = mainWindowViewModel};
        }

        public virtual void ShowMainWindow( MainWindowView mainWindowView ) {
            mainWindowView.Show( );
        }

        public void ApplicationExit( object sender, ExitEventArgs e ) {
            if ( this._mainWindowViewModel != null ) { this._mainWindowViewModel.CloseWindow( ); }
        }

        #endregion
    }
}
