﻿namespace QuickAccess.GraphicsManager.Commands {
    #region Imported Namespaces

    using System;
    using System.Collections.Generic;
    using Messages;
    using EventAggregator;

    #endregion

    public class CommandManager : ICommandManager {
        protected readonly Dictionary<string, IRelayCommand> _commandDictionary;

        protected IEventAggregator _eventAggregator;

        #region Implementation of IListener<CommandMessage>

        public bool Handle( CommandMessage message ) {
            if ( this._commandDictionary.ContainsKey( message.CommandToExecute ) ) {
                this._commandDictionary[ message.CommandToExecute ].Execute( message.ExecuteParameter );
            }
            return true;
        }

        #endregion

        #region Members

        private void OnCommandPressed( object obj ) {
            this._eventAggregator.Publish( (CommandMessage) obj );
        }

        #endregion

        #region Constructors

        public CommandManager( IEventAggregator eventAggregator ) {
            this._eventAggregator = eventAggregator;
            this._commandDictionary = new Dictionary<string, IRelayCommand>( );

            this.ManagerCommand = new RelayCommand( this.OnCommandPressed );

            this._eventAggregator.Subscribe( this );
        }

        public CommandManager( ) : this( EventAggregatorFactory.Instance ) {}

        #endregion

        #region Implementation of ICommandManager

        public IRelayCommand ManagerCommand { get; private set; }

        public bool Register( string commandName, IRelayCommand command ) {
            if ( !this._commandDictionary.ContainsKey( commandName ) ) {
                this._commandDictionary.Add( commandName, command );
                return true;
            }

            return false;
        }

        public bool DeRegister( string commandName ) {
            if ( this._commandDictionary.ContainsKey( commandName ) ) {
                this._commandDictionary.Remove( commandName );
                return true;
            }

            return false;
        }

        #endregion

        #region Implementation of IDisposable

        public void Dispose( ) {
            this.OnDispose( true );
            GC.SuppressFinalize( this );
        }

        #region Members

        protected virtual void OnDispose( bool onDispose ) {
            if ( onDispose ) {
                this._eventAggregator.Unsubscribe( this );
            }
        }

        #endregion

        #endregion
    }
}
