﻿namespace QuickAccess.EventAggregator {
    #region Imported Namespaces

    using System;
    using System.Collections.Generic;
    using System.Linq;

    #endregion

    public class EventAggregator : IEventAggregator {
        protected Dictionary<Type, IList<IListener>> _handlers = new Dictionary<Type, IList<IListener>>( );

        #region IEventAggregator Members

        public bool Subscribe<TMessage>( IListener<TMessage> listener ) where TMessage : IEventMessage {
            var messageType = typeof(TMessage);
            lock (this._handlers) {
                if ( this._handlers.ContainsKey( messageType ) ) {
                    if ( !this._handlers[ messageType ].Contains( listener ) || this._handlers[ messageType ].FirstOrDefault( l => ReferenceEquals( l, listener ) ) == null )
                    {
                        this._handlers[ messageType ].Add( listener );
                    }
                    return true;
                }

                this._handlers.Add( messageType, new List<IListener> {listener} );
                return true;
            }
        }

        public bool Unsubscribe<TMessage>( IListener<TMessage> listener ) where TMessage : IEventMessage {
            var messageType = typeof(TMessage);
            lock (this._handlers) {
                if ( !this._handlers.ContainsKey( messageType ) ) { return false; }

                return this._handlers[ messageType ].Contains( listener ) && this._handlers[ messageType ].Remove( listener );
            }
        }

        public bool UnsubscribeAll<TMessage>( ) where TMessage : IEventMessage {
            var messageType = typeof(TMessage);

            lock (this._handlers) { return this._handlers.ContainsKey( messageType ) && this._handlers.Remove( messageType ); }
        }

        public void Publish<TMessage>( TMessage message ) where TMessage : IEventMessage {
            var messageType = typeof(TMessage);

            lock (this._handlers) {
                if ( !this._handlers.ContainsKey( messageType ) ||
                     ( this._handlers[ messageType ].Count <= 0 ) ) { return; }
                if ( this._handlers[ messageType ].Cast<IListener<TMessage>>( ).Any( typedReference => typedReference.Handle( message ) ) ) { }
            }
        }

        #endregion
    }
}
