﻿namespace QuickAccess.GraphicsManager.Commands {
    #region Imported Namespaces

    using System;
    using System.Diagnostics.CodeAnalysis;

    #endregion

    public class RelayCommand : IRelayCommand {
        private readonly Predicate<object> _canExecute;

        private readonly Action<object> _execute;

        #region Constructors

        public RelayCommand( Action<object> execute, Predicate<object> canExecute = null ) {
            if ( execute == null ) { throw new ArgumentNullException( "execute" ); }

            this._execute = execute;
            this._canExecute = canExecute;
        }

        #endregion

        #region IRelayCommand Members

        public void Execute( object parameter ) {
            this._execute( parameter );
        }

        public bool CanExecute( object parameter ) {
            return this._canExecute == null || this._canExecute( parameter );
        }

        public event EventHandler CanExecuteChanged
        {
            add { System.Windows.Input.CommandManager.RequerySuggested += value; }
            remove { System.Windows.Input.CommandManager.RequerySuggested -= value; }
        }

        #endregion
    }
}
