﻿namespace QuickAccess.GraphicsManager.ViewModels {
    #region Imported Namespaces

    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Windows.Controls;
    using Commands;
    using ViewModelFramework;
    using QuickAccess.EventAggregator;

    #endregion

    public class MainWindowViewModel : BaseViewModel, IMainWindowViewModel {
        protected ObservableCollection<IButtonViewModel> _applicationMenuButtonViewModels;

        private IPageViewModel _currentlyActivePage;

        protected ObservableCollection<IPageViewModel> _pageViewModels;

        private string _programWindowName;

        protected bool _showTabHeaderCloseButton;

        private string _statusBarMessage;

        protected Dock _tabBarPosition;

        protected ObservableCollection<ITabViewModel> _tabViewModels;

        protected IEventAggregator _eventAggregator;

        #region Constructors

        public MainWindowViewModel( string programName,
            IEnumerable<ITabViewModel> tabViewModels,
            IEnumerable<IPageViewModel> pageViewModels,
            IEnumerable<IButtonViewModel> applicationMenuButtonViewModels,
            IRelayCommand buttonDirectorCommand,
            IEventAggregator eventAggregator )
        {
            // Set Fields
            this._programWindowName = programName;
            this._tabViewModels = new ObservableCollection<ITabViewModel>( tabViewModels );
            this._pageViewModels = new ObservableCollection<IPageViewModel>( pageViewModels );
            this._applicationMenuButtonViewModels = null;// applicationMenuButtonViewModels//new ObservableCollection<IButtonViewModel>( applicationMenuButtonViewModels );
            this.ButtonDirectorCommand = buttonDirectorCommand;
            this._eventAggregator = eventAggregator;
        }

        #endregion

        #region IMainWindowViewModel Members

        public string ProgramWindowName
        {
            get { return this._programWindowName; }
            set
            {
                this._programWindowName = value;
                this.OnPropertyChanged( );
            }
        }

        public ObservableCollection<IPageViewModel> PageViewModels
        {
            get { return this._pageViewModels; }
            set
            {
                this._pageViewModels = value;
                this.OnPropertyChanged( );
            }
        }

        public ObservableCollection<ITabViewModel> TabViewModels
        {
            get { return this._tabViewModels; }
            set
            {
                this._tabViewModels = value;
                this.OnPropertyChanged( );
            }
        }

        public ObservableCollection<IButtonViewModel> ApplicationMenuButtonViewModels
        {
            get { return this._applicationMenuButtonViewModels; }
            set
            {
                this._applicationMenuButtonViewModels = value;
                this.OnPropertyChanged( );
            }
        }

        public IRelayCommand ButtonDirectorCommand { get; set; }

        public IPageViewModel OverlayPageViewModel { get; set; }

        public IPageViewModel CurrentlyActivePage
        {
            get { return this._currentlyActivePage; }
            set
            {
                this._currentlyActivePage = value;
                this.OnPropertyChanged( );
            }
        }

        public Guid SelectedOperator { get; set; }

        public bool Initialized { get; private set; }

        public Dock TabBarPosition
        {
            get { return this._tabBarPosition; }
            set
            {
                this._tabBarPosition = value;
                this.OnPropertyChanged( );
            }
        }

        public bool ShowTabHeaderCloseButton
        {
            get { return this._showTabHeaderCloseButton; }
            set
            {
                this._showTabHeaderCloseButton = value;
                this.OnPropertyChanged( );
            }
        }

        public string StatusBarMessage
        {
            get { return this._statusBarMessage; }
            set
            {
                this._statusBarMessage = value;
                this.OnPropertyChanged( );
            }
        }

        public event EventHandler UpdateDefaultSelectedTab;

        protected virtual void OnUpdateDefaultSelectedTab( EventArgs e ) {
            EventHandler handler = this.UpdateDefaultSelectedTab;
            if ( handler != null ) { handler( this, e ); }
        }

        public bool Initialize( ) {
            if ( this.Initialized ) { return true; }

            // Set default standard items
            this._tabBarPosition = Dock.Top;
            this._showTabHeaderCloseButton = true;

            // Set commands

            this.SetCloseCommand( );
            this.SetPageFocusManagerCommand( );

            this.Initialized = true;

            return true;
        }

        public bool CloseWindow( ) {
            return true;
        }

        #endregion

        #region Members

        private void SetPageFocusManagerCommand( ) {
        }

        protected void SetCloseCommand( ) {
        }

        protected void SetActivePage( ) {
        }

        protected void SetActivePage( IPageViewModel iPageViewModel ) {
        }

        #endregion
    }
}
