﻿namespace QuickAccess.GraphicsManager.ViewModels {
    #region Imported Namespaces

    using System.Collections.Generic;
    using Commands;
    using Models;
    using ViewModelFramework;

    #endregion

    public class PageViewModel : BaseDomainObjectViewModel<IPage>, IPageViewModel {
        #region Implementation of IPageViewModel

        private IList<string> _boundTabs;

        private bool _closeButtonVisibility;

        private IRelayCommand _closeCommand;

        private string _pageTitle;

        private bool _preventPageClosing;

        private string _defaultTab;

        private IRelayCommand _managerCommand = CommandManagerFactory.Instance.ManagerCommand;

        #region Constructors

        public PageViewModel( IPage domainObject, string pageTitle, bool closeButtonVisibility ) : base( domainObject ) {
            this._pageTitle = pageTitle;
            this._closeButtonVisibility = closeButtonVisibility;
            this._preventPageClosing = false;
            this._boundTabs = new List<string>( );
        }

        #endregion

        public string PageTitle
        {
            get { return this._pageTitle; }
            set
            {
                this._pageTitle = value;
                this.OnPropertyChanged( );
            }
        }

        public bool CloseButtonVisibility
        {
            get { return this._closeButtonVisibility; }
            set
            {
                this._closeButtonVisibility = value;
                this.OnPropertyChanged( );
            }
        }

        public IRelayCommand CloseCommand
        {
            get { return this._closeCommand; }
            set
            {
                this._closeCommand = value;
                this.OnPropertyChanged( );
            }
        }

        public IRelayCommand ManagerCommand
        {
            get { return this._managerCommand; }
            set { this._managerCommand = value; }
        }

        public bool PreventPageClosing
        {
            get { return this._preventPageClosing; }
            set
            {
                this._preventPageClosing = value;
                this.OnPropertyChanged( );
            }
        }

        public IList<string> BoundTabs
        {
            get { return this._boundTabs; }
            set
            {
                this._boundTabs = value;
                this.OnPropertyChanged( );
            }
        }

        public string DefaultTab
        {
            get { return this._defaultTab; }
            set
            {
                this._defaultTab = value;
                this.OnPropertyChanged( );
            }
        }

        #endregion
    }
}
