﻿
namespace QuickAccess.GraphicsManager.Views {
    #region Imported Namespaces

    using System;
    using System.ComponentModel;
    using System.Linq;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using System.Windows.Media;
    using Infragistics.Windows.Controls;
    using Infragistics.Windows.Controls.Events;
    using Infragistics.Windows.Ribbon;
    using Infragistics.Windows.Themes;
    using ViewModels;
    using Messages;
    using Properties;

    #endregion

    public partial class MainWindowView {
        #region Constructors

        public MainWindowView( ) {
            this.InitializeComponent( );
            Application.Current.MainWindow = this;
        }

        #endregion

        private void RibbonMenu_Opened( object sender, RoutedEventArgs e ) {
            var ribbonContextMenu = sender as RibbonContextMenu;
            if ( ribbonContextMenu != null )
                ribbonContextMenu.Items.Clear( );
        }

        private void Selector_OnSelectionChanged( object sender, SelectionChangedEventArgs e ) {
            var pageViewModel = ( (XamTabControl) sender ).SelectedItem as IPageViewModel;

            this.SetSelectedTab( pageViewModel );
        }

        protected virtual void SetSelectedTab( IPageViewModel pageViewModel ) {
            if ( pageViewModel != null &&
                 !string.IsNullOrEmpty( pageViewModel.DefaultTab ) ) {
                var ribbonTabItem = this.HostRibbon.TabsInView.SingleOrDefault( x => ( (ITabViewModel) x.DataContext ).Identifier == pageViewModel.DefaultTab );
                this.HostRibbon.SelectedTab = ribbonTabItem;
            }
        }

        // TODO: Replace this functionality with an attached behavior! THIS IS BAD! MVVM VIOLATION!

        #region Members

        protected override void OnSourceInitialized( EventArgs e ) {
            base.OnSourceInitialized( e );
        }

        private void BuildRibbon( object sender, RoutedEventArgs e ) {
            IMainWindowViewModel vm = this.DataContext as MainWindowViewModel;

            if ( vm == null ) { return; }

            // Set theme
            ThemeManager.CurrentTheme = "Office2013";

            foreach ( var tabViewModel in vm.TabViewModels ) {
                if (tabViewModel is IContextualTabViewModel)
                {
                    this.HostRibbon.ContextualTabGroups.Add(GetContextualTabItem(vm, tabViewModel as IContextualTabViewModel));
                }
                else
                {
                    RibbonTabItem item = GetRibbonTabItem(vm, tabViewModel);
                    this.HostRibbon.Tabs.Add(item);
                }
            }

            // Set binding for selecting necessary tab
            vm.UpdateDefaultSelectedTab += this.UpdateDefaultSelectedTabHandler;
        }

        private void UpdateDefaultSelectedTabHandler( object sender, EventArgs e ) {
            var vm = this.DataContext as IMainWindowViewModel;
            if ( vm == null )
                return;

            var pageViewModel = vm.CurrentlyActivePage;
            if ( pageViewModel == null ) { return; }

            this.SetSelectedTab( pageViewModel );
        }

        private ContextualTabGroup GetContextualTabItem( IMainWindowViewModel vm, IContextualTabViewModel tabViewModel ) {
            var contextualTabGroup = new ContextualTabGroup {Caption = tabViewModel.Name, IsVisible = tabViewModel.IsVisible, BaseBackColor = Colors.White};
            var isVisibleBinding = new Binding( "IsVisible" ) {Source = tabViewModel, Mode = BindingMode.OneWay};
            BindingOperations.SetBinding( contextualTabGroup, ContextualTabGroup.IsVisibleProperty, isVisibleBinding );

            contextualTabGroup.Tabs.Add( GetRibbonTabItem( vm, tabViewModel ) );
            return contextualTabGroup;
        }

        private RibbonTabItem GetRibbonTabItem( IMainWindowViewModel vm, ITabViewModel tabViewModel ) {
            var ribbonTabItem = new RibbonTabItem {DataContext = tabViewModel, Header = tabViewModel.Name, ToolTip = tabViewModel.Description};
            // FIX: Use Tooltips instead
            foreach ( var buttonGroup in tabViewModel.ButtonGroups ) {
                var iButtonGroup = new RibbonGroup {DataContext = buttonGroup, Caption = buttonGroup.Name, ToolTip = buttonGroup.Description};
                // FIX: Use Tooltips instead
                RibbonGroup.SetMaximumSize( iButtonGroup, RibbonToolSizingMode.ImageAndTextLarge );
                foreach ( var buttonViewModel in buttonGroup.ButtonViewModels ) {
                    var buttonTool = new ButtonTool
                    {
                        Caption = buttonViewModel.Name,
                        LargeImage = buttonViewModel.Icon,
                        ToolTip = buttonViewModel.ShortDescription,
                        Command = vm.ButtonDirectorCommand,
                        CommandParameter = new CommandMessage
                        {
                            CommandToExecute = buttonViewModel.OnCallFunction,
                            ExecuteParameter = buttonViewModel.OnCallFunctionParameter
                        },
                        Id = buttonViewModel.Id
                    };


                    //if( !buttonViewModel.NotInRibbon )
                    //{
                        RibbonGroup.SetMaximumSize( buttonTool, RibbonToolSizingMode.ImageAndTextLarge );
                        iButtonGroup.Items.Add( buttonTool );
                    //}
                    //else
                    //{
                    //    // Add the capture button to tools not in ribbon
                    //    this.HostRibbon.ToolsNotInRibbon.Add( buttonTool );
                    //}
                }
                ribbonTabItem.RibbonGroups.Add( iButtonGroup );
            }

            return ribbonTabItem;
        }

        private void FocusChanged( object sender, RoutedEventArgs e ) {
            var vm = this.DataContext as IMainWindowViewModel;
            var pageViewModel = ( (TabItemEx) sender ).DataContext as IPageViewModel;
            if ( vm == null ||
                 pageViewModel == null ) { return; }

            this.SetSelectedTab( pageViewModel );
        }

        private void TabClosed( object sender, TabClosingEventArgs e ) {
        }

        private void MainWindowView_OnClosing( object sender, CancelEventArgs e ) {
            var vm = this.DataContext as IMainWindowViewModel;
            if ( vm == null ) { return; }

            var result = vm.CloseWindow( );
            e.Cancel = !result;

            if ( e.Cancel ) { return; }

            Settings.Default.Save( );
        }

        #endregion

        private static bool once;
        private void HostRibbon_OnLoaded( object sender, RoutedEventArgs e ) {

            Dispatcher.BeginInvoke(new Action(() =>
            {
                XamRibbon ribbon = sender as XamRibbon;

                if (!once)
                {
                    once = true;
                    return;
                }

                //ButtonTool tool = new ButtonTool() { Id = "ScreenCapture", Caption = "Screen Cap" };
                //ribbon.ToolsNotInRibbon.Add(tool);

                QatPlaceholderTool btnBoldPH = new QatPlaceholderTool("ScreenCapture");
                btnBoldPH.TargetId = "ScreenCapture";

                ButtonTool tool = ribbon.Tabs[0].RibbonGroups[0].Items[0] as ButtonTool;
                ribbon.Tabs[0].RibbonGroups[0].Items.Clear();
                ribbon.ToolsNotInRibbon.Add(tool);

                ribbon.QuickAccessToolbar.Items.Add(btnBoldPH);
            }));
        }
    }
}
